@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2023 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2023 Open Text.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -149,7 +149,8 @@ class RowDescription(BackendMessage):
         return self.fields
 
     def __str__(self):
-        return "RowDescription: {}".format(self.fields)
+        s = ",\n".join([c.debug_info() for c in self.fields])
+        return f"RowDescription: [\n{s}]"
 
 
 BackendMessage.register(RowDescription)
