@@ -58,7 +58,8 @@ class Startup(BulkFrontendMessage):
     message_id = None
 
     def __init__(self, user, database, session_label, os_user_name, autocommit,
-                 binary_transfer, request_complex_types):
+                 binary_transfer, request_complex_types, oauth_access_token,
+                 workload, auth_category):
         BulkFrontendMessage.__init__(self)
 
         try:
@@ -95,8 +96,14 @@ class Startup(BulkFrontendMessage):
             b'binary_data_protocol': '1' if binary_transfer else '0', # Defaults to text format '0'
             b'protocol_features': '{"request_complex_types":' + request_complex_types + '}',
             b'protocol_compat': 'VER',
+            b'workload': workload,
+            b'auth_category': auth_category,
         }
 
+        if len(oauth_access_token) > 0:
+            # compatibility for protocol version 3.11
+            self.parameters[b'oauth_access_token'] = oauth_access_token
+
     def read_bytes(self):
         # The fixed protocol version is followed by pairs of parameter name and value strings.
         # A zero byte is required as a terminator after the last name/value pair.
